// LANGUAGE.H
//
// List of Command & Status Signal Names, their values, and their Bit Masks
//
// This has NO RELEVANCE to the scantool. I'll fix it up later.
//
// Nov. 24, 1995
//
//


//  C-	COMMAND FORMATS

#define C_Reset					0x80  // 10000000 B	Reset Controller State Machine, ACB, & Biofilter to State 0
#define C_Reset_MASK			0x83  // 10000011 B	Masks are applied to leave the 'Don't Care' bits alone 

#define C_AutoSequence			0x81  // 10000001 B	Autosequence Initiate: Advances Controller State Machine
#define C_AutoSequence_MASK		0x83  // 10000011 B from 0 to 1 (READY State)

#define C_Set_Biofilter			0x8A  // 10001010 B	Biofilter Flip Flop Set/Reset
#define C_Reset_Biofilter		0x82  // 10000010 B	
#define C_Biofilter_MASK		0x8F  // 10001111 B

#define C_Set_ACB				0x8E  // 10001110 B Annular Confinement Beam Flip Flop Set/Reset
#define C_Reset_ACB				0x86  // 10000110 B
#define C_ACB_MASK				0x8F  // 10001111 B

#define C_Scan					0x83  // 10000011 B	Prepares the Transporter Controller to
#define C_Scan_MASK				0x83  // 10000011 B	send or receive Scan Data - see Scan Data Formats, below...


//  D-	 DATA FORMATS

#define D_Read_STATE_FF			0x00  // 00000000 B	Read Status Word
#define D_Read_STATE_FF_MASK	0x8C  // 10001100 B	(prepare to read state FF on Done/Error bits)

#define D_Read_ACBnBio_FF		0x04  // 00000100 B	Read Status Word
#define D_Read_ACBnBio_FF_MASK	0x8C  // 10001100 B	(prepare to read state ACB and Biofilter FF on Done/Error bits)

#define D_Scan_Data_00			0x08  // 00001000 B	Scan Data
#define D_Scan_Data_01			0x09  // 00001001 B these are the four units of data, 
#define D_Scan_Data_10			0x0A  // 00001010 B each individually named, if I feel like using them.  
#define D_Scan_Data_11			0x0B  // 00001011 B
#define D_Scan_Data_Cmd 		0x08  // 00001000 B General Scan Data command to use instead of discrete values above
#define D_Scan_Data_Data_MASK	0x03  // 00000011 B A mask to for the actual scan data for use with above Command
#define D_Scan_Data_MASK		0x8F  // 10001111 B

#define D_End_of_Data			0x0C  // 00001100 B	End - of - Data Word
#define D_End_of_Data_MASK		0x8C  // 10001100 B	Indicates to the controller that the Process is Complete


// CB- Checkbits - Evertyhing we need to know to use EDAC with the controller

#define CB_Num_Databits			4		// Indicates the Number of Databits involved in the Error Detection And Correction
#define CB_Data_LSB_Pos			0		// The 0 - Based value indicating the bit position of the
//											first databit (LSB) for EDAC use
#define CB_Check_LSB_POS		4		// 7-x***xxxx-0	The 0 - Based value indicating the bit position of the
//														first checkbit (LSB) for EDAC use
#define CB_Checkbit_MASK		0x70	// 01110000 B	Mask for Checkbits of the Byte used for Controller Communications
#define CB_Databit_MASK			0x0F	// 00001111 B	Mask for Databits of the Byte used for Controller Communications


//  S-	SIGNALS (returned from Controller)

#define S_DONE_MASK				0x20  // 00100000 B	Mask for DONE Signal Bit of Controller Status Byte
#define S_DONE_Data				0x00  // 00000000 B	DONE signal to be Negated after 'Scan Data' or 'End-of-Data' sent
#define S_DONE_Command			0x20  // 00100000 B	DONE signal to be Asserted after any Command is sent


#define S_ERROR_MASK			0x10  // 00010000 B	Mask for ERROR Signal Bit of Controller Status Byte
#define S_ERROR					0x10  // 00010000 B	ERROR signal to be Asserted by Controller under
//													the following conditions:
//
//													Command Sent (D7 = 1) AND
//													Controller is in State 1 (FF0=1 & FF1=0) AND
//													Materialize Operation (S7 = 1 (is UP) ) AND
//													Biofilter FF is not set (is 0)

//													OR

//													Command Sent (D7 = 1) AND
//													Controller is in State 1 (FF0=1 & FF1=0) AND
//													Annular Confinement Beam FF is not set (is 0)

//													OR

//													Virus Is Detected


//	SIGNALS (returned from controller upon 'Read Status' Request)

#define S_STATE_FF_MASK			0x30  // 00110000 B	Mask for Controller State Machine Flip Flops 0 and 1 Status Bits

#define S_STATE_FF_0			0x00  // 00000000 B	State of Controller State Machine is State 0
#define S_STATE_FF_1			0x20  // 00100000 B	State of Controller State Machine is State 1
#define S_STATE_FF_2			0x10  // 00010000 B	State of Controller State Machine is State 2
#define S_STATE_FF_3			0x30  // 00110000 B	State of Controller State Machine is State 3


// (I'm not sure yet if I'll test individual flip flops, or test both at once, so I'll include definitions for both cases) 


#define S_STATE_FF0_MASK		0x20  // 00100000 B	Mask for Controller State Machine Flip Flop #0 Status Bit
#define S_STATE_FF1_MASK		0x10  // 00010000 B	Mask for Controller State Machine Flip Flop #1 Status Bit

#define S_STATE_FF0_SET			0x20  // 00100000 B	State of Controller State Machine Flip Flop #0
#define S_STATE_FF0_RESET		0x00  // 00000000 B	(SET = 1, RESET = 0)

#define S_STATE_FF1_SET			0x10  // 00010000 B	State of Controller State Machine Flip Flop #0
#define S_STATE_FF1_RESET		0x00  // 00000000 B	(SET = 1, RESET = 0)



#define S_Biofilter_FF_MASK		0x10  // 00010000 B	Mask for Biofilter Flip Flop Status Bit of Controller Status Byte
#define S_Biofilter_FF_SET		0x10  // 00010000 B	State of Biofilter Flip Flop (SET = 1, RESET = 0)
#define S_Biofilter_FF_RESET	0x00  // 00000000 B

#define S_ACB_FF_MASK			0x20  // 00100000 B	Mask for ACB Flip Flop Status Bit of Controller Status Byte
#define S_ACB_FF_SET			0x20  // 00100000 B	State of ACB Flip Flop (SET = 1, RESET = 0)
#define S_ACB_FF_RESET			0x00  // 00000000 B

// End of File
