/******************************************************************************
 st.cpp
	
	The Seantool
	GM C3I 160 bits/second ALDL serial data via Parallel (Printer) Port

	Created February 1, 1999

	Copyright  1999 by Seanster <Seanster@BrainImplants.com>
	All rights reserved.

 ******************************************************************************/

// INCLUDES

#include <String.h>			// String Functions
#include <iostream.h>		// Stream Functions for Debug Output        
#include <iomanip.h>		// Stream Functions for Debug Output        
#include <Math.h>		// Stream Functions for Debug Output        
//#include <mmsystem.h>		// Sound Support



#include "pio.h"
#include "language.h"		// Names for all of the Bits & Bytes used to communicate with
							// the Controller Chip (hence 'language')
//#include "sounds.h"			// Defines relating Sound Titles to .wav filenames.
#include "typedefs.h"       // Project Global Defines and Typedefs

#include "st.h"				// Class definition


// DEFINES

	// Debugging output switch
#define DEBUG	true


// GLOBALS

int Sound = ON;				// Sound ON/OFF programmer's switch


//	Seantool Member Functions


// Object Constructor

Seantool::Seantool()
{

myPIO = new PIO;		// Create a Parallel IO Port Object for Communicaton to/from Controller

}

// Object Destructor  - Called when Object is to be Deleted

Seantool::~Seantool()
{

delete myPIO;			// Delete our Parallel I/O Object

}


/*******************************************************************
// Calibrate
//
//		A few things:
//
//		1.  Determine the value of the two states for the IO port
			(easy, just read for a 1/2 second)
//		2.  Assign the two states to HIGH and LOW
			(easy, low is the one that has the shortest life)
//		3.  Determine the distance (aka time) from the edge trigger
//			to valid data.
			(easy, wait for edge trigger. Loop until you get
			a HIGH. Do that A few times & average them.
			Multiply that average by 5 and there you are,
			right in the middle of valid data.
			(about 2 milliseconds from start of cycle (the falling edge)))
//
//		The rest are a wishlist:
//
//		4.  Determine the number of bits in a 'byte'
			(Listen and count the largest # of consecutive
			 high bytes (would be, oh.. no more than 20 probably)
			 Then loop from there to some reasonably
			 smallest byte size, say 5 or so.
			Loop: look for the start byte
			and check that a few bytes afterwards do not also
			look like start bytes?, decrement)
//		5.  Determine the number of bytes in a frame
		    (just count the # of bytes between start bytes.. simple)
//
//		Returns TRUE if calibration is successful.
//
 *******************************************************************/

bool Seantool::Calibrate()
{
bool ret = true;

		// these may seem bass-ackwards because I invert the aldl output
		// voltage into the computer (in order to make the interface trivial)

		// note that we only actually use LOW_VALUE, the other is irrelevant

	HIGH_VALUE	= 255;	// integer value of port status representing logical '1'
	LOW_VALUE	= 127;	// integer value of port status representing logical '0'

	byte_length		= 9;	// # of bits in a byte (9 on my car)

	frame_length	= 26;	// # of bytes in a frame (26 on my car)

	delay_loops		= 2300;	// after the edge trigger, delay # of loops then read the data

		
		// storage for all the bytes in a frame

	frame_data	= new int(frame_length +1);
	

//	itsStatus = itsPIO->Transfer( itsMessage );

#ifdef DEBUG
	cout << "Calibrate:" << endl;
	cout << "	byte_length:  " << byte_length  << endl;
	cout << "	frame_length: " << frame_length << endl;
	cout << "	delay_loops:  " << delay_loops  << endl;
#endif

	/*
	cout << "delay_loops = ?";
	cin >> delay_loops;
	cout << endl << "delay_loops = " << delay_loops << endl;
	*/	

	return ret;
}


//*******************************************************************
// Run
//
//		Returns TRUE if calibration is successful.
//
//*******************************************************************
bool Seantool::Run()
{
bool	ret			= true;
bool	state;				// current value HIGH/LOW on the wire
bool	last_state	= LOW;	// to prevent us starting on a false trigger
int		byte_value	= 0;	// the byte we build by adding bits to it
int		byte_bits	= 0;	// number of bits collected into the current byte
int		frame_bytes	= 0;	// # of bytes collected into the frame
int		frame_bits	= 0;	// >0 means # of bits read into the current frame, 0=no frame
int		skip		= 0;	// ??
int		start_bits	= 0;	// number of consecutive HIGH bits
int		frames		= 0;	// number of frames decoded
int		notriggers	= 0;	// # of iterations without a trigger found
int		frame_array[27];


//	if ( Sound == ON ) sndPlaySound (SND_process_blip, SND_SYNC);

#ifdef DEBUG
	cout << endl << "Processing..." << endl;
#endif


		// Loop until we are satisfied
	do
	{
			// Read HIGH/LOW from the data port

		state = (myPIO->Read() == HIGH_VALUE);
		
			// check for a falling edge
		
		if ( (state == HIGH) && (last_state == LOW) )
		{
			// okay.. we had a falling edge, data will appear soon

			notriggers = 0; // reset counter


				// delay an exact time until data becomes valid
			for (int i = 0; i < delay_loops + 1; i++)
			{
				state = (myPIO->Read() == HIGH_VALUE);
			}

			// Okay, so we cheated and did one extra loop to count
			// as the actual data fetch
			// That one loop could be important on slow systems

			byte_bits++;		// we have another bit to add to our byte
			
			if ( state == LOW )
			{
				//PRINT " 0";

				// if we have a zero bit, then obviously we do NOT
				// have the start of a byte here and we reset the counter

				start_bits = 0;
			}
			else	// status == HIGH
			{
				//PRINT " 1";

					// add the bit into the byte
					// the first bit is always ignored
					// as it is the 'start bit' and has no bearing on the value
					// the bits arrive in order from MSB to LSB
				if ( byte_bits > 1 ) byte_value += (int) pow (2,(byte_length - byte_bits));
				
				start_bits++;	// we track the # of consecutive HIGH bits
					
				if ( start_bits == byte_length )
				{
						// we counted enough consecutive start bits to know
						// that this is the byte that starts the frame

						// the start byte is all HIGH bits
						// so we set that value here
						// it's not already set as expected because we 
						// normally ignore the start bit
					byte_value = (int) pow (2,(byte_length + 1)) - 1;

					start_bits = 0;	//start looking for consecutive start bits again

					byte_bits  = byte_length;  // so we know to print out this value
						
						// we will +1 right below here to make up for the -1
					frame_bits = byte_length -1;  // start byte is a byte in the frame

					//cout << "F" << setw( 2 ) << frames + 1 << ">";	// start of frame
				}
			}

				
			// if we are not synced into a valid frame,
			// the byte collected is irrelevant

			if ( frame_bits > 0 )
			{
				// we are in a valid frame

				frame_bits++;	// increment the # of bits in this frame

				if (byte_bits == byte_length) // && frame_bits > byte_length)
				{
						// we are in a valid frame and we collected a whole byte

						// we can prevent the useless start byte from printing
						// if we also check for frame_bits > byte_length

					//cout << "(FB" << frame_bits << ")";

					frame_bytes++;

					//cout << " " << setw( 3 ) << byte_value << flush; //output the byte value
					frame_array[frame_bytes-1] = byte_value;

					//if ( frame_bytes == 16 ) cout << endl;

					byte_value = 0;		// reset the value
					byte_bits = 0;		// reset the counter

				}
				
				//cout << "frame_bits " << frame_bits << " fl*bl " << (frame_length * byte_length);

				if ( frame_bits == (frame_length * byte_length) )
				{
						// we have reached the end of a valid frame

					frames++;	// chalk up another frame deciphered

					cout << "F" << setw( 2 ) << frames + 1 << ">" << setw( 3 );
					
					for (int i = 0; i < frame_length; i++)
					{
						if ( i == 16 ) cout << endl << " ";

						cout << setw(4) << frame_array[i];
					}
					
					cout << endl << endl;	// print a carriage return\linefeed
					


						// reset until another valid start of frame is detected
					frame_bits = 0;
					frame_bytes = 0;	// reset counter
				}
			} // frame
		} //edge trigger
		else
		{
			notriggers++; // increment counter

			/*
			if ( (notriggers % 1000) == 0)
			{
				cout << "notriggers = " << notriggers << endl;
			}
			*/
			if ( notriggers > delay_loops * 18 )
			{
				// We waited FAR too long for a falling edge
				// increase samp

				cout << endl << "No triggers even after " << notriggers << " delay_loops. New value? ";
				cin >> delay_loops;
				cout << endl << "delay_loops = " << delay_loops << endl;
				
				notriggers = 0; // now start over
			}
		}
				

			// remember the status so we can detect the falling edge
		last_state = state;
		
	if (frames > 60 )
	{
		cout << endl << "5 Frames located, delay_loops = ? ";
		cin >> delay_loops;
		cout << endl << "delay_loops = " << delay_loops << endl;

		notriggers = 0; // now start over
		frames = 0;
	}

	} while ( frames < 100 ); // attempt to prevent us from looping forever, fix up later

	cout << endl << endl;	// tidy up the screen

	return ret;
}

// st.cpp


/*
	byte_value = 0;
   for (byte_bits = 2; byte_bits < 10; byte_bits++)
   {
	   byte_value += (int) pow (2,(byte_length - byte_bits));
   
	   cout << "bb, bv " << byte_bits << " " << byte_value << endl;
   }

	return true;
*/