/******************************************************************************
 st.h
	
	The Seantool
	GM C3I 160 bits/second ALDL serial data via Parallel (Printer) Port

	Created February 1, 1999

	Copyright  1999 by Seanster <Seanster@BrainImplants.com>
	All rights reserved.

 ******************************************************************************/

#ifndef _ST_H
#define _ST_H

// string table definitions

#include "typedefs.h"

class PIO;

class Seantool
{

private:

		// My definition of BIT states. These are defined as:
		// HIGH = 1, LOW = 0
		// LOW is the same voltage as the 0.5 msec state always
		// present at the end of the cycle defining a BIT

		// Therefore the HIGH is the voltage at the bottom of the falling
		// edge that must occur at the beginning of the cycle 
		// (right after the aforementioned mandatoy LOW state)

		// ('falling' edge would make more sense if it was falling from HIGH to
		// LOW logically, rather than in terms of voltage)

		// Ok.. just to be certain about this: the byte which marks
		// the start of the frame is considered to have each of it's
		// bits ALL HIGH. No other byte can possibly have this.
		// The first bit on all other bytes is LOW.

		// This is just my way of looking at it. If you have your own
		// view (PWM for example) then you can probably figure this out anyway.

	int		HIGH_VALUE;	// integer value of port status representing logical '1'
	int		LOW_VALUE;	// integer value of port status representing logical '0'

		// logical '1'
	#define HIGH	true

		// logical '0'
	#define LOW		false


	int		delay_loops;	// after the edge trigger, delay # of loops then read the data


		// the first bit in a byte is usually LOW.
		// only one byte is allowed to have the first bit HIGH.

	int		byte_length;	// # of bits in a byte (9 on my car)


		// the one byte with all bits HIGH marks
		// the start of a frame of bytes AND IS INCLUDED IN THE FRAME
		// very important to remember!

	int		frame_length;	// # of bytes in a frame (26 on my car)
	int		*frame_data;	// array with [frame_length] elements
//	int		**frames_array[];	// frame storage array
	
public:

	PIO  *myPIO;       // The Parallel Port Object to Send our Bytes

	  Seantool();	// Constructor
	 ~Seantool();	// Destructor


    // Talk to Scantool

	virtual bool Calibrate(void);
	virtual bool Run(void);

};

#endif //_ST_H
